@@ -3,7 +3,7 @@ BEGIN {
   $POE::Quickie::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $POE::Quickie::VERSION = '0.07';
+  $POE::Quickie::VERSION = '0.08';
 }
 
 use strict;
@@ -45,9 +45,7 @@ sub _create_session {
                 _stop
                 _exception
                 _create_wheel
-                _delete_wheel
                 _child_signal
-                _child_closed
                 _child_timeout
                 _child_stdout
                 _child_stderr
@@ -117,7 +115,6 @@ sub _create_wheel {
     my $wheel;
     eval {
         $wheel = POE::Wheel::Run->new(
-            CloseEvent  => '_child_closed',
             StdoutEvent => '_child_stdout',
             StderrEvent => '_child_stderr',
             Program     => $program,
@@ -145,10 +142,16 @@ sub _create_wheel {
     $self->{wheels}{$wheel->ID}{args} = $args;
     $self->{wheels}{$wheel->ID}{alive} = 2;
 
+    if ($args->{Input}) {
+        $wheel->put($args->{Input});
+        $wheel->shutdown_stdin();
+    }
+
     if (defined $args->{Timeout}) {
         $self->{wheels}{$wheel->ID}{alrm}
             = $kernel->delay_set('_child_timeout', $args->{Timeout}, $wheel->ID);
     }
+
     $kernel->sig_child($wheel->PID, '_child_signal');
 
     return (undef, $wheel);
@@ -166,15 +169,31 @@ sub _exception {
 sub _child_signal {
     my ($kernel, $self, $pid, $status) = @_[KERNEL, OBJECT, ARG1, ARG2];
     my $id = $self->_pid_to_id($pid);
+
     $self->{wheels}{$id}{status} = $status;
     $self->{lazy}{$pid}{status} = $status if $self->{lazy}{$pid};
-    $kernel->yield('_delete_wheel', $id);
-    return;
-}
 
-sub _child_closed {
-    my ($kernel, $self, $id) = @_[KERNEL, OBJECT, ARG0];
-    $kernel->yield('_delete_wheel', $id);
+    my $s = $status >> 8;
+    if ($s != 0 && !exists $self->{wheels}{$id}{args}{ExitEvent}) {
+        warn "Child $pid exited with status $s\n";
+    }
+
+    my $event   = $self->{wheels}{$id}{args}{ExitEvent};
+    my $context = $self->{wheels}{$id}{args}{Context};
+
+    $kernel->alarm_remove($self->{wheels}{$id}{alrm});
+    delete $self->{wheels}{$id};
+
+    if (defined $event) {
+        $kernel->post(
+            $self->{parent_id},
+            $event,
+            $status,
+            $pid,
+            (defined $context ? $context : ()),
+        );
+    }
+
     return;
 }
 
@@ -254,40 +273,6 @@ sub _child_stderr {
     return;
 }
 
-# only delete the wheel after both child_signal and child_closed
-# have called this
-sub _delete_wheel {
-    my ($kernel, $self, $id) = @_[KERNEL, OBJECT, ARG0];
-
-    $self->{wheels}{$id}{alive}--;
-    return if $self->{wheels}{$id}{alive};
-
-    my $pid = $self->{wheels}{$id}{obj}->PID;
-    my $status = $self->{wheels}{$id}{status};
-    my $s = $status >> 8;
-    if ($s != 0 && !exists $self->{wheels}{$id}{args}{ExitEvent}) {
-        warn "Child $pid exited with status $s\n";
-    }
-
-    my $event   = $self->{wheels}{$id}{args}{ExitEvent};
-    my $context = $self->{wheels}{$id}{args}{Context};
-
-    $kernel->alarm_remove($self->{wheels}{$id}{alrm});
-    delete $self->{wheels}{$id};
-
-    if (defined $event) {
-        $kernel->post(
-            $self->{parent_id},
-            $event,
-            $status,
-            $pid,
-            (defined $context ? $context : ()),
-        );
-    }
-
-    return;
-}
-
 sub _pid_to_id {
     my ($self, $pid) = @_;
 
@@ -490,6 +475,10 @@ is false.
 B<'ProgramArgs'> (optional), same as the epynomous parameter to
 POE::Wheel::Run.
 
+B<'Input'> (optional), a string containing the input to the program. This
+string, if provided, will be sent immediately to the program, and its
+stdin will then be shut down.
+
 B<'StdoutEvent'> (optional), the event for delivering lines from the
 program's STDOUT. If you don't supply this, they will be printed to the main
 program's STDOUT. To explicitly ignore them, set this to C<undef>.
